/*  Copyright (C) 2012-2022 by László Nagy
    This file is part of Bear.

    Bear is a tool to generate compilation database for clang tooling.

    Bear is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bear is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

// specific platform indicator, should not using like this. (needs to be deprecated)
#define SUPPORT_PRELOAD
/* #undef SUPPORT_MULTILIB */

// header checks
#define HAVE_SPAWN_H
#define HAVE_UNISTD_H
#define HAVE_DLFCN_H
#define HAVE_ERRNO_H
#define HAVE_SYS_UTSNAME_H
#define HAVE_SYS_WAIT_H
#define HAVE_SYS_TIME_H
#define HAVE_SYS_STAT_H
#define HAVE_GNU_LIB_NAMES_H

// OSX specific (needs to be deprecated)
/* #undef HAVE_NSGETENVIRON */

// dynamic linker
#define HAVE_DLOPEN
#define HAVE_DLSYM
#define HAVE_DLERROR
#define HAVE_DLCLOSE
#define HAVE_RTLD_NEXT

// errno values
#define HAVE_EACCES
#define HAVE_ENOENT
#define HAVE_UNAME

// string.h function
#define HAVE_STRERROR_R

// unistd.h might declare the environ symbol
#define HAVE_ENVIRON

// confstr and macros
#define HAVE_CONFSTR
#define HAVE_CS_PATH
#define HAVE_CS_GNU_LIBC_VERSION
#define HAVE_CS_GNU_LIBPTHREAD_VERSION

// fmt version
#define HAVE_FMT_STD_H
#define FMT_NEEDS_OSTREAM_FORMATTER

// macros to disable compiler generated methods
#define NON_DEFAULT_CONSTRUCTABLE(T) \
    T() noexcept = delete;

#define NON_COPYABLE_NOR_MOVABLE(T) \
    T(T const &) = delete; \
    T& operator=(T const &) = delete; \
    T(T &&) noexcept = delete; \
    T& operator=(T &&) noexcept = delete;

// const expression about this project
namespace cmd {
    constexpr char VERSION[] = "3.0.20";

    namespace bear {

        constexpr char FLAG_CITNAMES[] = "--citnames";
        constexpr char FLAG_INTERCEPT[] = "--interceptor";
    }

    namespace citnames {
        constexpr char DEFAULT_PATH[] = "/usr/bin/citnames";

        constexpr char FLAG_INPUT[] = "--input";
        constexpr char FLAG_OUTPUT[] = "--output";
        constexpr char FLAG_APPEND[] = "--append";
        constexpr char FLAG_RUN_CHECKS[] = "--run-checks";
        constexpr char FLAG_CONFIG[] = "--config";

        constexpr char DEFAULT_OUTPUT[] = "compile_commands.json";
    }

    namespace intercept {
        constexpr char DEFAULT_PATH[] = "/usr/bin/intercept";

        constexpr char FLAG_OUTPUT[] = "--output";
        constexpr char FLAG_LIBRARY[] = "--library";
        constexpr char FLAG_WRAPPER[] = "--wrapper";
        constexpr char FLAG_WRAPPER_DIR[] = "--wrapper-dir";
        constexpr char FLAG_COMMAND[] = "--";
        constexpr char FLAG_FORCE_WRAPPER[] = "--force-wrapper";
        constexpr char FLAG_FORCE_PRELOAD[] = "--force-preload";

        constexpr char DEFAULT_OUTPUT[] = "events.json";
    }

    namespace wrapper {
        constexpr char DEFAULT_PATH[] = "/usr/lib64/bear/wrapper";
        constexpr char DEFAULT_DIR_PATH[] = "/usr/lib64/bear/wrapper.d";

        constexpr char FLAG_VERBOSE[] = "--verbose";
        constexpr char FLAG_DESTINATION[] = "--destination";
        constexpr char FLAG_EXECUTE[] = "--execute";
        constexpr char FLAG_COMMAND[] = "--";

        constexpr char KEY_DESTINATION[] = "INTERCEPT_REPORT_DESTINATION";
        constexpr char KEY_VERBOSE[] = "INTERCEPT_VERBOSE";
    }

    namespace library {

#ifdef SUPPORT_MULTILIB
        // Multilib is one of the solutions allowing users to run applications
        // built for various application binary interfaces (ABIs) of the same
        // architecture. The most common use of multilib is to run 32-bit
        // applications on 64-bit kernel.
        //
        // For Linux, a small tune is needed at build time. Need to compile
        // `libexec.so` library for 32-bit and for 64-bit too. Then install
        // these libraries to the OS preferred multilib directories.
        // And use the `libexec.so` path default value with a single path,
        // that matches both. (The match can be achieved by the $LIB token
        // expansion from the dynamic loader. See `man ld.so` for more.)
        constexpr char DEFAULT_PATH[] = "/usr/$LIB/bear/libexec.so";
#else
        constexpr char DEFAULT_PATH[] = "/usr/lib64/bear/libexec.so";
#endif
        constexpr char KEY_REPORTER[] = "INTERCEPT_REPORT_COMMAND";
        constexpr char KEY_DESTINATION[] = "INTERCEPT_REPORT_DESTINATION";
        constexpr char KEY_VERBOSE[] = "INTERCEPT_VERBOSE";
    }
}

constexpr char OS_DIR_SEPARATOR = '/';
constexpr char OS_PATH_SEPARATOR = ':';
